/*-
 * Copyright 2023 Hochschule für Technik Stuttgart
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.hft.stuttgart.quality.adapter.types;

import javax.xml.namespace.QName;

import org.xmlobjects.annotation.XMLElement;
import org.xmlobjects.builder.ObjectBuildException;
import org.xmlobjects.serializer.ObjectSerializeException;
import org.xmlobjects.stream.XMLReadException;
import org.xmlobjects.stream.XMLReader;
import org.xmlobjects.stream.XMLWriteException;
import org.xmlobjects.stream.XMLWriter;
import org.xmlobjects.xml.Attributes;
import org.xmlobjects.xml.Element;
import org.xmlobjects.xml.Namespaces;

import de.hft.stuttgart.quality.QualityADEModule;
import de.hft.stuttgart.quality.model.types.SemanticAttributeInvalidError;

@XMLElement(name = "SE_ATTRIBUTE_INVALID", namespaceURI = QualityADEModule.NAMESPACE_URI)
public class SemanticAttributeInvalidErrorAdapter extends AbstractSemanticErrorAdapter<SemanticAttributeInvalidError> {

	@Override
	public SemanticAttributeInvalidError createObject(QName name, Object parent) throws ObjectBuildException {
		return new SemanticAttributeInvalidError();
	}

	@Override
	public void buildChildObject(SemanticAttributeInvalidError object, QName name, Attributes attributes,
			XMLReader reader) throws ObjectBuildException, XMLReadException {
		if (!QualityADEModule.NAMESPACE_URI.equals(name.getNamespaceURI())) {
			return;
		}
		switch (name.getLocalPart()) {
		case "childId":
			reader.getTextContent().ifPresent(object::setChildId);
			break;
		case "attributeName":
			reader.getTextContent().ifPresent(object::setAttributeName);
			break;
		default:
			throw new IllegalStateException(
					"Cannot handle name " + name + " when building SemanticAttributeInvalidError element");
		}
	}

	@Override
	public Element createElement(SemanticAttributeInvalidError object, Namespaces namespaces)
			throws ObjectSerializeException {
		return Element.of(QualityADEModule.NAMESPACE_URI, "SE_ATTRIBUTE_INVALID");
	}

	@Override
	public void writeChildElements(SemanticAttributeInvalidError object, Namespaces namespaces, XMLWriter writer)
			throws ObjectSerializeException, XMLWriteException {
		if (object.getChildId() != null) {
			writer.writeElement(
					Element.of(QualityADEModule.NAMESPACE_URI, "childId").addTextContent(object.getChildId()));
		}
		if (object.getAttributeName() != null) {
			writer.writeElement(Element.of(QualityADEModule.NAMESPACE_URI, "attributeName")
					.addTextContent(object.getAttributeName()));
		}
	}
}
