package de.hft.stuttgart.quality.adapter.types;

import javax.xml.namespace.QName;

import org.citygml4j.xml.adapter.core.AbstractFeatureAdapter;
import org.xmlobjects.annotation.XMLElement;
import org.xmlobjects.builder.ObjectBuildException;
import org.xmlobjects.serializer.ObjectSerializeException;
import org.xmlobjects.stream.XMLReadException;
import org.xmlobjects.stream.XMLReader;
import org.xmlobjects.stream.XMLWriteException;
import org.xmlobjects.stream.XMLWriter;
import org.xmlobjects.xml.Attributes;
import org.xmlobjects.xml.Element;
import org.xmlobjects.xml.Namespaces;

import de.hft.stuttgart.quality.QualityADEModule;
import de.hft.stuttgart.quality.adapter.properties.FilterPropertyAdapter;
import de.hft.stuttgart.quality.adapter.properties.GlobalParametersPropertyAdapter;
import de.hft.stuttgart.quality.adapter.properties.RequirementPropertyAdapter;
import de.hft.stuttgart.quality.model.properties.RequirementProperty;
import de.hft.stuttgart.quality.model.types.ValidationPlan;

@XMLElement(name = "ValidationPlan", namespaceURI = QualityADEModule.NAMESPACE_URI)
public class ValidationPlanAdapter extends AbstractFeatureAdapter<ValidationPlan> {

	@Override
	public ValidationPlan createObject(QName name, Object parent) throws ObjectBuildException {
		return new ValidationPlan();
	}

	@Override
	public void buildChildObject(ValidationPlan object, QName name, Attributes attributes, XMLReader reader)
			throws ObjectBuildException, XMLReadException {
		if (!QualityADEModule.NAMESPACE_URI.equals(name.getNamespaceURI())) {
			return;
		}
		switch (name.getLocalPart()) {
		case "globalParameters":
			object.setGlobalParameters(reader.getObjectUsingBuilder(GlobalParametersPropertyAdapter.class));
			break;
		case "filter":
			object.setFilter(reader.getObjectUsingBuilder(FilterPropertyAdapter.class));
			break;
		case "requirement":
			object.getRequirements().add(reader.getObjectUsingBuilder(RequirementPropertyAdapter.class));
			break;
		default:
			super.buildChildObject(object, name, attributes, reader);
		}
	}

	@Override
	public Element createElement(ValidationPlan object, Namespaces namespaces) throws ObjectSerializeException {
		return Element.of(QualityADEModule.NAMESPACE_URI, "ValidationPlan");
	}

	@Override
	public void writeChildElements(ValidationPlan object, Namespaces namespaces, XMLWriter writer)
			throws ObjectSerializeException, XMLWriteException {
		if (object.getGlobalParameters() != null) {
			writer.writeElementUsingSerializer(Element.of(QualityADEModule.NAMESPACE_URI, "globalParameters"),
					object.getGlobalParameters(), GlobalParametersPropertyAdapter.class, namespaces);
		}
		if (object.getFilter() != null) {
			writer.writeElementUsingSerializer(Element.of(QualityADEModule.NAMESPACE_URI, "filter"), object.getFilter(),
					FilterPropertyAdapter.class, namespaces);
		}
		for (RequirementProperty req : object.getRequirements()) {
			writer.writeElementUsingSerializer(Element.of(QualityADEModule.NAMESPACE_URI, "requirement"), req,
					RequirementPropertyAdapter.class, namespaces);
		}
	}
}
